<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       http://themetrust.com
 * @since      1.0.0
 *
 * @package    Proven
 * @subpackage Proven/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Proven
 * @subpackage Proven/admin
 * @author     ThemeTrust <info@themetrust.com>
 */
class Proven_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;
	
	/**
	 * Error messages.
	 *
	 * @since 1.0.0
     * @var array $errors
	 */
    static public $errors = array();

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 */
	public function __construct( $plugin_name, $version ) {
		$this->plugin_name = $plugin_name;
		$this->version = $version;
	}
	
	/**
     * Registers a post type.
     *
     * @since 1.0.0
     * @return void
     */
    static public function register_post_type()
    {
        $labels = array(
            'name'                => 'Proven',
            'singular_name'       => 'Proven',
            'add_new'             => esc_html__( 'Add New', 'tt-proven' ) ,
            'add_new_item'        => esc_html__( 'Add New', 'tt-proven' ),
            'edit_item'           => esc_html__( 'Edit', 'tt-proven' ),
            'new_item'            => esc_html__( 'New', 'tt-proven' ),
            'view_item'           => esc_html__( 'View', 'tt-proven' ),
            'search_items'        => esc_html__( 'Search', 'tt-proven' ),
            'not_found'           => esc_html__( 'No Proven found', 'tt-proven' ),
            'not_found_in_trash'  => esc_html__( 'No Proven found in Trash', 'tt-proven' ),
            'menu_name'           => 'Proven',
        );
        $args = array(
            'labels'              => $labels,
            'hierarchical'        => false,
            'description'         => '',
            'taxonomies' 		  => array( '' ),
            'public'              => false,
            'show_ui'             => true,
            'show_in_menu'        => true,
            'show_in_admin_bar'   => true,
            'menu_position'       => 80,
            'menu_icon'           => 'dashicons-yes',
            'show_in_nav_menus'   => true,
            'publicly_queryable'  => false,
            'exclude_from_search' => true,
            'has_archive'         => false,
            'query_var'           => true,
            'can_export'          => true,
            'rewrite'             => '',
            'capability_type'     => 'post',
            'supports'            => array( 'title' ),
        );

        register_post_type( 'tt_proven', $args );
        flush_rewrite_rules();
		require_once TT_PROVEN_DIR . 'includes/metaboxes.php';
    }

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/proven-admin.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {
			wp_enqueue_script( 'tt-proven-admin-script', plugin_dir_url( __FILE__ ) . 'js/proven-admin.js', array( 'jquery' ), $this->version, false );
			wp_localize_script( 'tt-proven-admin-script', 'tt_proven', array(
            	'messages'  => array(
                	'mailchimp_connect_success'   => __('Successfully fetched MailChimp data.', 'tt-proven'),
					'mailchimp_connect_label'   => __('Refresh Data', 'tt-proven')
            			)
        	));
	}
	
	public function add_plugin_admin_menu() {

	    /*
	     * Add a settings page for this plugin to the Settings menu.
	     *
	     * NOTE:  Alternative menu locations are available via WordPress administration menu functions.
	     *
	     *        Administration Menus: http://codex.wordpress.org/Administration_Menus
	     *
	     */
	    
	    add_submenu_page( 'edit.php?post_type=tt_proven', 'Settings', 'Settings', 'administrator', 'proven-settings', 'Proven_Admin::display_plugin_settings_page' );
	
	}
	
	static public function page_request() {
		if ( isset( $_REQUEST['page'] ) && 'proven-settings' == $_REQUEST['page'] ) {
            self::update_settings();
        }
	}
	
	/**
	 * Admin notices.
	 *
	 * @since 1.0.0
	 * @return void
	 */
	static public function admin_notices()
	{
		if ( ! is_admin() ) {
			return;
		}
		else if ( ! is_user_logged_in() ) {
			return;
		}
		else if ( ! current_user_can( 'update_core' ) ) {
			return;
		}

		if ( count( self::$errors ) ) {
			foreach ( self::$errors as $key => $msg ) {
				?>
				<div class="notice notice-error">
					<p><?php echo $msg; ?></p>
				</div>
				<?php
			}
		}
	}
	
	/**
     * Render admin post columns.
     *
     * @since 1.0.0
     */
    static public function add_post_custom_column( $columns )
    {
		$columns['tt-proven-active'] = __('Active', 'tt-proven');
        return apply_filters( 'tt_proven_post_columns', $columns );
    }

   	/**
    * Render content for admin post columns.
    *
    * @since 1.0.0
    */
    static public function manage_post_custom_column( $column, $post_id )
    {
        switch ( $column ) {
           
            case 'tt-proven-active':
                $active = get_post_meta( $post_id, '_tt_proven_active', true );

                self::notification_toggle_control( $active, $post_id );
                break;
        }

        do_action( 'tt_proven_post_columns_content', $column, $post_id );
    }

    static public function notification_toggle_control( $active = '1', $post_id )
    {
	
        $text           = __('Active', 'tt-proven');
        $img_active     = TT_PROVEN_URL . 'admin/img/toggle_active.png';
        $img_inactive   = TT_PROVEN_URL . 'admin/img/toggle_inactive.png';
        $img            = $img_active;

        if ( ! $active ) {
            $text   = __('Inactive', 'tt-proven');
            $img    = $img_inactive;
        }
        ?>
        <img src="<?php echo $img; ?>" class="tt-proven-toggle" alt="<?php echo $text; ?>" title="<?php echo $text; ?>" data-nonce="<?php echo wp_create_nonce('tt_proven_active_nonce'); ?>" data-post="<?php echo $post_id; ?>" />
        <?php
    }

	 /**
	 * Add settings action link to the plugins page.
	 *
	 * @since    1.0.0
	 */

	static public function add_action_links( $links ) {
	    
	}

	/**
	 * Render the settings page for this plugin.
	 *
	 * @since    1.0.0
	 */

	static public function display_plugin_settings_page() {
		$mailchimp_connect_label = __('Connect', 'tt_proven');
		$mailchimp_api_key = get_option( 'tt_proven_mailchimp_api_key' );
		$lists = get_option( 'tt_proven_mailchimp_lists' );
		if ( $lists && ! empty( $lists ) && $mailchimp_api_key && ! empty( $mailchimp_api_key ) ) {
            $mailchimp_connect_label = __('Refresh Data', 'tt_proven');
        }

	    include_once( 'partials/admin-proven-settings.php' );
	}
	
	/**
	 * Render the update message.
	 *
	 * @since 1.0.0
	 */
    static public function render_update_message()
    {
        if ( ! empty( $_POST ) && ! isset( $_POST['email'] ) ) {
			echo '<div class="updated"><p>' . esc_html__( 'Successfully updated!', 'tt-proven' ) . '</p></div>';
		}
    }

	/**
	 * Render the action for the form.
	 *
	 * @since 1.0.0
	 */
    static public function get_form_action( $query_var = '' ) {
        $page = '/edit.php?post_type=tt_proven&page=proven-settings';

        if ( is_network_admin() ) {
		    return network_admin_url( $page . $query_var );
		}
		else {
			return admin_url( $page . $query_var );
		}
    }

	public function register_admin_settings() {
		//register our settings
		register_setting( 'tt-proven-settings-group', 'tt_proven_license_key' );
		register_setting( 'tt-proven-settings-group', 'tt_proven_disable_credit_link' );
		register_setting( 'tt-proven-settings-group', 'tt_proven_mailchimp_api_key' );
		register_setting( 'tt-proven-settings-group', 'tt_proven_mailchimp_lists' );
		register_setting( 'tt-proven-settings-group', 'tt_proven_disable_credit' );
	}
	
	/**
	 * Update settings.
	 *
	 * @since 1.0.0
	 * @return void
	 */
    static private function update_settings()
    {
        if ( ! is_admin() ) {
            return;
        }

        // Only admins can save settings.
		if ( ! current_user_can('manage_options') ) {
			return;
		}

        // Verify nonce.
        if ( ! isset( $_POST['tt_proven_settings_nonce'] ) || ! wp_verify_nonce( $_POST['tt_proven_settings_nonce'], '_tt_proven_settings_nonce' ) ) {
            return;
        }

        // Save license.
        if ( isset( $_POST['tt_proven_license_key'] ) ) {
            $license_key = tt_proven_sanitize_license( $_POST['tt_proven_license_key'] );
            update_option( 'tt_proven_license_key', esc_attr( $license_key ) );
        }

		// Save MailChimp API Key.
        if ( isset( $_POST['tt_proven_mailchimp_api_key'] ) ) {
            $mailchimp_api = $_POST['tt_proven_mailchimp_api_key'];
            update_option( 'tt_proven_mailchimp_api_key', esc_attr( $mailchimp_api ) );
        }

        // Save credit link.
        if ( isset( $_POST['tt_proven_disable_credit'] ) ) {
            update_option( 'tt_proven_disable_credit', '1' );
        } else {
            delete_option( 'tt_proven_disable_credit' );
        }

        
    }

	public function ajax_headers() {
        send_origin_headers();
		@header( 'Content-Type: text/html; charset=' . get_option( 'blog_charset' ) );
		@header( 'X-Robots-Tag: noindex' );
		send_nosniff_header();
		nocache_headers();
		status_header( 200 );
    }


	/**
     * Toggles status for custom post type columns.
     *
     * @since 1.0.0
     */
    public function tt_proven_toggle_active() {
        self::ajax_headers();

        $error = false;

        if ( ! isset( $_POST['nonce'] ) || ! wp_verify_nonce( $_POST['nonce'], 'tt_proven_active_nonce' ) ) {
            $error = true;
        }

        if ( ! isset( $_POST['post_id'] ) || empty( $_POST['post_id'] ) || ! absint( $_POST['post_id'] ) ) {
            $error = true;
        }

        if ( $error ) {
            echo __('There was an error.', 'tt-proven');
            die();
        }

        $post_id = absint( $_POST['post_id'] );
        $active = $_POST['status'] == 'active' ? '1' : '0';

        update_post_meta( $post_id, '_tt_proven_active', $active );

        echo 'success';
        die();
    }

	static public function tt_proven_render_meta_boxes() {
		global $pagenow;

		    // If current page is post.php and post isset than query for its post type 
		    // if the post type is 'event' do something
		    if ( 'post.php' === $pagenow && isset($_GET['post']) && 'tt_proven' === get_post_type( $_GET['post'] ) ){
		        // Do something
		    }
    		require_once TT_PROVEN_DIR . 'includes/metaboxes.php';
		
	}

	function remove_unwanted_meta_boxes() {
		remove_meta_box( 'mymetabox_revslider_0', 'tt_proven', 'normal' );
	}
	
	/**
	 * Admin Preview.
	 *
	 * @since 1.0.0
	 * @return void
	 */
    static public function tt_proven_admin_preview()
    {
        global $pagenow;

        if ( ! in_array( $pagenow, array( 'post.php', 'post-new.php' ) ) ) {
            return;
        }

        global $post_type;

        if ( $post_type == "tt_proven" ) {

            $post_id = isset( $_REQUEST['post'] ) ? $_REQUEST['post'] : '';

            if ( empty( $post_id ) ) {
                global $post;
                $post_id = $post->ID;
            }


            include TT_PROVEN_DIR . 'admin/partials/admin-proven-preview.php';

            do_action( 'tt_proven_admin_preview' );
        }
    }
	

}
