<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       http://themetrust.com
 * @since      1.0.0
 *
 * @package    Proven
 * @subpackage Proven/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Proven
 * @subpackage Proven/includes
 * @author     Henry Jones <info@themetrust.com>
 */
class Proven {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Proven_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'PLUGIN_NAME_VERSION' ) ) {
			$this->version = PLUGIN_NAME_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'proven';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Proven_Loader. Orchestrates the hooks of the plugin.
	 * - Proven_i18n. Defines internationalization functionality.
	 * - Proven_Admin. Defines all hooks for the admin area.
	 * - Proven_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-proven-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-proven-i18n.php';
		
		/**
		 * The class for helper functions.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-proven-helper.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-proven-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-proven-public.php';
		
		/**
		 * The class responsible for helper functions.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-proven-helper.php';
		
		/**
		 * The class responsible for Woocommerce integration.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-woocommerce.php';
		
		/**
		 * The class responsible for Easy Digital Downloads integration.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-edd.php';
		
		/**
		 * The class responsible for MailChimp integration.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-mailchimp.php';
		
		/**
		 * License Updater.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/updater/edd_sl_updater_config.php';

		$this->loader = new Proven_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Proven_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Proven_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Proven_Admin( $this->get_plugin_name(), $this->get_version() );
		$tt_proven_mailchimp = new TT_Proven_MailChimp();

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
		
		// Add menu item
		$this->loader->add_action( 'admin_menu', $plugin_admin, 'add_plugin_admin_menu' );
		
		// Register Post Type
		$this->loader->add_action( 'init',  $plugin_admin, 'register_post_type' );
		
		// Register admin settings
		$this->loader->add_action( 'admin_init', $plugin_admin, 'register_admin_settings' );
		$this->loader->add_action( 'admin_init', $plugin_admin, 'page_request' );
	
		// Add custom columns
		$this->loader->add_filter( 'manage_tt_proven_posts_columns', $plugin_admin, 'add_post_custom_column' );
        $this->loader->add_action( 'manage_tt_proven_posts_custom_column', $plugin_admin, 'manage_post_custom_column', 10, 2 );

		// Toggle active status for custom post type table
		$this->loader->add_action( 'wp_ajax_tt_proven_toggle_active', $plugin_admin, 'tt_proven_toggle_active' );
		
		// Render meta boxes
		//$this->loader->add_action( 'admin_init', $plugin_admin, 'tt_proven_render_meta_boxes' );
		
		// Remove unwanted meta boxes
		$this->loader->add_action( 'do_meta_boxes', $plugin_admin, 'remove_unwanted_meta_boxes' );
		
		// Render the admin preview
		$this->loader->add_action( 'admin_footer', $plugin_admin, 'tt_proven_admin_preview' );
		
		//MailChimp
		$this->loader->add_action( 'wp_ajax_tt_proven_connect_mailchimp', $tt_proven_mailchimp, 'tt_proven_connect_mailchimp' );
		$this->loader->add_action( 'save_post', $tt_proven_mailchimp, 'fetch_subscriber_cron_job' );
        
	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Proven_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );
		$this->loader->add_action( 'wp_footer', $plugin_public, 'render_proven' );
		$this->loader->add_action( 'wp', $plugin_public, 'get_active_items' );
		$this->loader->add_action( 'wp_ajax_show_items', $plugin_public, 'show_items' );
		$this->loader->add_action( 'wp_ajax_nopriv_show_items', $plugin_public, 'show_items' );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Proven_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
