<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       http://themetrust.com
 * @since      1.0.0
 *
 * @package    Proven
 * @subpackage Proven/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Proven
 * @subpackage Proven/public
 * @author     Henry Jones <info@themetrust.com>
 */
class Proven_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;
	
	public static $active_items = array();
    public static $is_cookie_set = false;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;
	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/proven-public.css', array(), $this->version, 'all' );
		wp_enqueue_style( 'dashicons' );

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {
		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/proven-public.js', array( 'jquery' ), $this->version, false );
	}
	
	/**
     * Get IP address of the client.
     *
     * @since 1.0.0
     * @return string
     */
    static public function get_client_ip() {
        $ipaddress = '';
        if (getenv('HTTP_CLIENT_IP'))
            $ipaddress = getenv('HTTP_CLIENT_IP');
        else if(getenv('HTTP_X_FORWARDED_FOR'))
            $ipaddress = getenv('HTTP_X_FORWARDED_FOR');
        else if(getenv('HTTP_X_FORWARDED'))
            $ipaddress = getenv('HTTP_X_FORWARDED');
        else if(getenv('HTTP_FORWARDED_FOR'))
            $ipaddress = getenv('HTTP_FORWARDED_FOR');
        else if(getenv('HTTP_FORWARDED'))
           $ipaddress = getenv('HTTP_FORWARDED');
        else if(getenv('REMOTE_ADDR'))
            $ipaddress = getenv('REMOTE_ADDR');
        else
            $ipaddress = 'UNKNOWN';
        return $ipaddress;
    }

	public function ajax_headers()
    {
        send_origin_headers();
		@header( 'Content-Type: text/html; charset=' . get_option( 'blog_charset' ) );
		@header( 'X-Robots-Tag: noindex' );
		send_nosniff_header();
		nocache_headers();
		status_header( 200 );
    }

	public function set_cookie()
    {
        $cookie_ip = ( isset( $_COOKIE['tt_proven_ip'] ) ) ? $_COOKIE['tt_proven_ip'] : '';
        if ( empty( $cookie_ip ) ) {
            $current_ip = self::get_client_ip();
            setcookie( 'tt_proven_ip', $current_ip, strtotime('+1 month'), "/" );
            self::$is_cookie_set = 1;
        }
    }

	static public function is_this_page( $pages ) {
		$pages_array = explode(",", $pages);
		$page_object = get_queried_object();
		$current_page_id = get_queried_object_id();
		foreach ( $pages_array as $page ) {
			if($page == $current_page_id){
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Get all active items.
	 *
	 * @since 1.0.0
	 */
    public function get_active_items()
    {
        $args = array(
            'post_type'         => 'tt_proven',
            'posts_per_page'    => '-1',
            'meta_query'        => array(
                array(
                    'key'           => '_tt_proven_active',
                    'value'         => '1',
                    'compare'       => '='
                )
            )
        );

        $posts = get_posts( $args );

        if ( count( $posts ) ) {
            foreach ( $posts as $post ) {
                self::$active_items[] = $post->ID;
            }
        }
    }

	/**
     * Show Items
     *
     * @since 1.0.0
     */
    public function show_items()
    {
        self::ajax_headers();

        if ( ! isset( $_POST['nonce'] ) || ! wp_verify_nonce( $_POST['nonce'], 'tt_proven_front_end_nonce' ) ) {
            return;
        }
        if ( ! isset( $_POST['ids'] ) || empty( $_POST['ids'] ) || ! is_array( $_POST['ids'] ) ) {
            return;
        }

        $ids = $_POST['ids'];
        $data = array();

        foreach ( $ids as $id ) {
	
            $start_delay = get_post_meta( $id, '_tt_proven_start_delay', true );
			$duration = get_post_meta( $id, '_tt_proven_duration', true );
			$between_delay = get_post_meta( $id, '_tt_proven_delay_between', true );
			$initial_delay = get_post_meta( $id, '_tt_proven_delay_initial', true );
			$animation     = get_post_meta( $id, '_tt_proven_animation', true );
			$max_number = get_post_meta( $id, '_tt_proven_max_number', true );
            $data[$id]['options'] = array(
                'id'                => $id,
                'start_delay'       => !empty( $start_delay ) ? $start_delay * 1000 : 0,
                'duration'          => !empty( $duration ) ? $duration * 1000 : 3000,
                'between_delay'     => !empty( $between_delay ) ? $between_delay * 1000 : 5000,
				'initial_delay'     => !empty( $initial_delay ) ? $initial_delay * 1000 : 5000,
				'animation'         => $animation,
                'max_per_page'      => absint( $max_number ),
            );

            ob_start();
            include 	TT_PROVEN_DIR . 'public/partials/proven-box.php';
            $content = ob_get_clean();
            $data[$id]['content'] = $content;
        }

        echo json_encode($data); die;
    }

	/**
     * Replaces content tags with values.
     *
     * @since 1.0.0
     */
    static public function proven_content_template( $content, $tags )
    {
        foreach ( $tags as $tag => $value ) {
            $content = str_replace( $tag, $value, $content );
        }
        return $content;
    }

	/**
     * Render Proven box.
     *
     * @since 1.0.0
     */
    static public function render_proven() {
        
        if ( empty( self::$active_items ) ) {
            return;
        }

        $proven_ids = array();

        foreach ( self::$active_items as $id ) {

            $user_logged_in        = is_user_logged_in();
            $user_type             = get_post_meta( $id, '_tt_proven_users', true );
			$show_on_pages         = get_post_meta( $id, '_tt_proven_show_on', true );
	        $show_on_these_pages   = get_post_meta( $id, '_tt_proven_show_on_these_pages', true );
	        $hide_on_these_pages   = get_post_meta( $id, '_tt_proven_hide_on_these_pages', true );
			$current_ip            = self::get_client_ip();
            $visitors              = get_post_meta( $id, '_tt_proven_visitors', true );
            $cookie_ip             = ( isset( $_COOKIE['tt_proven_ip'] ) ) ? $_COOKIE['tt_proven_ip'] : '';
			$position              = get_post_meta( $id, '_tt_proven_position', true );
			$shape                 = get_post_meta( $id, '_tt_proven_shape', true );
			$background_color      = get_post_meta( $id, '_tt_proven_background_color', true );
			$text_color            = get_post_meta( $id, '_tt_proven_text_color', true );
			$text_color_first_line = get_post_meta( $id, '_tt_proven_text_color_first_line', true );
			$link_color            = get_post_meta( $id, '_tt_proven_link_color', true );
			

            // Check if user meets user logged in/out settings.
            if ( ( $user_logged_in && $user_type == 'logged-out' ) || ( !$user_logged_in && $user_type == 'logged-in' ) ) {
                continue;
            }

            if ( $show_on_pages == 'all' ) {
				if ( !empty( $hide_on_these_pages ) && self::is_this_page( $hide_on_these_pages ) ) {
                    continue;
                }
            }elseif ( $show_on_pages == 'certain' ) {
				if ( !empty( $show_on_these_pages ) && !self::is_this_page( $show_on_these_pages ) ) {
                    continue;
                }
            }

			// Visitors
            $cookie_ip  = ( isset( $_COOKIE['tt_proven_ip'] ) ) ? $_COOKIE['tt_proven_ip'] : '';
            
            if ( $visitors == 'new' ) {
                if ( $cookie_ip == $current_ip && ! self::$is_cookie_set ) {
                    continue;
                }
            }
			
			?>

			<style id="tt-proven-style-<?php echo $id; ?>">
			    #tt-proven-holder-<?php echo $id; ?> .tt-proven-box {
			        <?php if( $background_color ) { ?>
			            background: <?php echo $background_color; ?>;
			        <?php } ?>
			        <?php if( $text_color ) { ?>
			            color: <?php echo $text_color; ?>;
			        <?php } ?>
			    }
				<?php if( $text_color_first_line ) { ?>
					#tt-proven-holder-<?php echo $id; ?> .tt-proven-box .tt-proven-text::first-line {
		            	color: <?php echo $text_color_first_line; ?>;
					}
		        <?php } ?>
			    #tt-proven-holder-<?php echo $id; ?> .tt-proven-box .tt-proven-close, #tt-proven-holder-<?php echo $id; ?> .tt-proven-box a.tt-proven-credit {
			        <?php if( $text_color ) { ?>
			            color: <?php echo $text_color; ?> !important;
			        <?php } ?>
			    }
			</style>
			
			<?php 
            $proven_ids[] = $id;

            
        }

        if ( !empty( $proven_ids ) ) {
            ?>
            <script type="text/javascript">
                var ttProven = {
                    nonce: '<?php echo wp_create_nonce('tt_proven_front_end_nonce'); ?>',
                    proven_ids: <?php echo json_encode( $proven_ids ); ?>,
                    ajaxurl: '<?php echo admin_url('admin-ajax.php'); ?>'
                };
            </script>
            <?php
        }
    }
	

}
